class ImageGeneratorException(Exception):
    pass


class ImageGeneratorWarning(Warning):
    pass


class InvalidAlignException(ImageGeneratorException):
    def __init__(self, align_provided, available_aligns=None):
        if available_aligns is None:
            available_aligns = ['center', 'right', 'left']
        available_aligns_str = ' or '.join(available_aligns)
        super().__init__(
            "Align has to be set either to {}. You provided '{}'".format(available_aligns_str, align_provided))


class FontNotFoundWarning(ImageGeneratorWarning):
    def __init__(self, fontpath_provided):
        super().__init__(
            "Font {} hasn't been found. Default font has been set instead".format(fontpath_provided))


class DefaultFontWarning(ImageGeneratorWarning):
    def __init__(self):
        super().__init__(
            "Font hasn't been provided. Default font has been set instead")


class ImageSizeFixedWarning(ImageGeneratorWarning):
    def __init__(self):
        super().__init__(
            "Image size has been adjusted to the provided content size because the content took too much space")



import math
import os
import warnings

from PIL import Image as PILImage
from PIL import ImageDraw

from nider.core import MultilineTextUnit
from nider.core import SingleLineTextUnit

from nider.utils import get_random_bgcolor
from nider.utils import get_random_texture
from nider.utils import is_path_creatable

from nider.colors import color_to_rgb

from nider.exceptions import ImageSizeFixedWarning

Header = MultilineTextUnit
Header.__doc__ = 'Base class for the header unit'
Paragraph = MultilineTextUnit
Paragraph.__doc__ = 'Base class for the paragraph unit'


class Linkback(SingleLineTextUnit):
    '''Class that represents a linkback used in images

    Attributes:
        bottom_padding (int): padding to step back from the bottom of the image.
    '''

    def __init__(self, bottom_padding=20, *args, **kwargs):
        self.bottom_padding = bottom_padding
        super().__init__(*args, **kwargs)

    def _set_height(self):
        '''Sets linkback\'s height'''
        super()._set_height()
        self.height += self.bottom_padding


class Content:
    '''Class that aggregates different units into a sigle object'''
    # Variable to check if the content fits into the img. Default is true,
    # but it may changed by in Img._fix_image_size()
    fits = True

    def __init__(self, paragraph, header=None, linkback=None, padding=45):
        self.para = paragraph
        self.header = header
        self.linkback = linkback
        self.padding = padding
        self._set_content_height()

    def _set_content_height(self):
        '''Sets content\'s height'''
        self.height = self.para.height + 2 * self.padding
        if self.header:
            self.height += 1 * self.padding + self.header.height
        if self.linkback:
            self.height += self.linkback.height


class Image:
    '''Base class for a text based image

    Attributes:
        content (nider.models.Content): object that has units to be rendered.
        fullpath (str): path where the image has to be saved.
        width (int): width of the image.
        height (int): height of the image.
    '''

    def __init__(self, content, fullpath, width=1080, height=1080):
        self._set_content(content)
        self._set_fullpath(fullpath)
        self._set_image_size(width, height)

    def draw_on_texture(self, texture_path=None):
        '''Draws preinitialized image and its attributes on a texture

        Draws preinitiated image and its attributes on a texture. If texture_path
        is set to None, takes random textures from textures/

        Attributes:
            texture_path (str): path of the texture to use.
        '''
        if texture_path is None:
            texture_path = get_random_texture()
        elif not os.path.isfile(texture_path):
            raise FileNotFoundError(
                'Can\'t find texture {}. Please, choose an existing texture'.format(texture_path))
        self._create_image()
        self._create_draw_object()
        self._fill_image_with_texture(texture_path)
        self._draw_content()
        self._save()

    def draw_on_bg(self, bgcolor=None):
        '''Draws preinitialized image and its attributes on a colored background

        Draws preinitiated image and its attributes on a colored background. If bgcolor
        is set to None, random nider.colors.colormap.FLAT_UI color is generated

        Attributes:
            bgcolor (str, tuple): either hex or rgb representation of background color.
        '''
        self.bgcolor = color_to_rgb(
            bgcolor) if bgcolor else get_random_bgcolor()
        self._create_image()
        self._create_draw_object()
        self._fill_image_with_color()
        self._draw_content()
        self._save()

    def draw_on_image(self, image_path):
        '''Draws preinitialized image and its attributes on an image

        Attributes:
            image_path (str): path of the image to draw on.
        '''
        if not os.path.isfile(image_path):
            raise FileNotFoundError(
                'Can\'t find image {}. Please, choose an existing image'.format(image_path))
        self.image = PILImage.open(image_path)
        self._create_draw_object()
        self.width, self.height = self.image.size
        self._draw_content()
        self._save()

    def _save(self):
        '''Saves the image'''
        self.image.save(self.fullpath)

    def _set_content(self, content):
        '''Sets content used in the image'''
        self.content = content
        self.header = content.header
        self.para = content.para
        self.linkback = content.linkback

    def _set_fullpath(self, fullpath):
        '''Sets path where to save the image'''
        if is_path_creatable(fullpath):
            self.fullpath = fullpath
        else:
            raise AttributeError(
                "Is seems impossible to create a file in path {}".format(fullpath))

    def _set_image_size(self, width, height):
        '''Sets width and height of the image'''
        if width <= 0 or height <= 0:
            raise AttributeError(
                "Width or height of the image have to be positive integers")
        self.width = width
        self.height = height

    def _fix_image_size(self):
        '''Fixes image's size'''

        if self.content.height >= self.height:
            warnings.warn(ImageSizeFixedWarning())
            self.content.fits = False
            self.height = self.content.height

    def _create_image(self):
        '''Creates a basic PIL image

        Creates a basic PIL image previously fixing its size
        '''
        self._fix_image_size()
        self.image = PILImage.new("RGBA", (self.width, self.height))

    def _create_draw_object(self):
        '''Creates a basic PIL Draw object'''
        self.draw = ImageDraw.Draw(self.image)

    def _fill_image_with_texture(self, texture_path):
        '''Fills an image with a texture

        Fills an image with a texture by reapiting it necessary number of times

        Attributes:
            texture_path (str): path of the texture to use
        '''
        texture = PILImage.open(texture_path, 'r')
        texture_w, texture_h = texture.size
        bg_w, bg_h = self.image.size
        times_for_Ox = math.ceil(bg_w / texture_w)
        times_for_Oy = math.ceil(bg_h / texture_h)
        for y in range(times_for_Oy):
            for x in range(times_for_Ox):
                offset = (x * texture_w, y * texture_h)
                self.image.paste(texture, offset)

    def _fill_image_with_color(self):
        '''Fills an image with a color

        Fills an image with a color by creating a colored rectangle of the image
        size
        '''
        self.draw.rectangle([(0, 0), self.image.size], fill=self.bgcolor)

    def _draw_content(self):
        '''Draws each unit of the content on the image'''
        if self.header:
            self._draw_header()
        if self.para:
            self._draw_para()
        if self.linkback:
            self._draw_linkback()

    def _draw_header(self):
        '''Draws the header on the image'''
        current_h = self.content.padding
        self._draw_unit(current_h, self.header)

    def _draw_para(self):
        '''Draws the paragraph on the image'''
        if self.content.fits:
            # Trying to center everything
            current_h = math.floor(
                (self.height - self.para.height) / 2)
            self._draw_unit(current_h, self.para)
        else:
            if self.header:
                header_with_padding_height = 2 * self.content.padding + self.header.height
                current_h = header_with_padding_height
            else:
                current_h = self.content.padding
            self._draw_unit(current_h, self.para)

    def _draw_linkback(self):
        '''Draws a linkback on the image'''
        current_h = self.height - self.linkback.height
        self._draw_unit(current_h, self.linkback)

    def _draw_unit(self, start_height, unit):
        '''Draws the text and its pseudo-shadow on the image starting at specific height'''
        current_h = start_height
        try:
            lines = unit.wrapped_lines
        except AttributeError:
            # text is a one-liner. Construct a list out of it for later usage
            lines = [unit.text]
        line_padding = getattr(unit, 'line_padding', None)
        for line in lines:
            w, h = self.draw.textsize(line, font=unit.font)
            if unit.align == "center":
                x = (self.width - w) / 2
            elif unit.align == "left":
                x = self.width * 0.075
            elif unit.align == "right":
                x = 0.925 * self.width - w
            y = current_h

            if unit.drop_shadow:
                # thin border
                self.draw.text((x - 1, y), line, font=unit.font,
                               fill=unit.shadowcolor)
                self.draw.text((x + 1, y), line, font=unit.font,
                               fill=unit.shadowcolor)
                self.draw.text((x, y - 1), line, font=unit.font,
                               fill=unit.shadowcolor)
                self.draw.text((x, y + 1), line, font=unit.font,
                               fill=unit.shadowcolor)

                # thicker border
                # self.draw.text((x - 1, y - 1), line, font=unit.font,
                #                fill=unit.shadowcolor)
                # self.draw.text((x + 1, y - 1), line, font=unit.font,
                #                fill=unit.shadowcolor)
                # self.draw.text((x - 1, y + 1), line, font=unit.font,
                #                fill=unit.shadowcolor)
                # self.draw.text((x + 1, y + 1), line, font=unit.font,
                #                fill=unit.shadowcolor)
                # end shadow

            self.draw.text((x, y), line, unit.color, font=unit.font)

            if line_padding:
                current_h += h + line_padding


class FacebookSquarePost(Image):
    '''Alias of models.Image with width=470 and height=470'''

    def __init__(self, *args, **kwargs):
        kwargs['width'] = kwargs.get('width', 470)
        kwargs['height'] = kwargs.get('height', 470)
        super().__init__(*args, **kwargs)


class FacebookLandscapePost(Image):
    '''Alias of models.Image with width=1024 and height=512'''

    def __init__(self, *args, **kwargs):
        kwargs['width'] = kwargs.get('width', 1024)
        kwargs['height'] = kwargs.get('height', 512)
        super().__init__(*args, **kwargs)


TwitterPost = FacebookLandscapePost


class TwitterLargeCard(Image):
    '''Alias of models.Image with width=506 and height=506'''

    def __init__(self, *args, **kwargs):
        kwargs['width'] = kwargs.get('width', 506)
        kwargs['height'] = kwargs.get('height', 506)
        super().__init__(*args, **kwargs)


class InstagramSquarePost(Image):
    '''Alias of models.Image with width=1080 and height=1080'''

    def __init__(self, *args, **kwargs):
        kwargs['width'] = kwargs.get('width', 1080)
        kwargs['height'] = kwargs.get('height', 1080)
        super().__init__(*args, **kwargs)


class InstagramPortraitPost(Image):
    '''Alias of models.Image with width=1080 and height=1350'''

    def __init__(self, *args, **kwargs):
        kwargs['width'] = kwargs.get('width', 1080)
        kwargs['height'] = kwargs.get('height', 1350)
        super().__init__(*args, **kwargs)


class InstagramLandscapePost(Image):
    '''Alias of models.Image with width=1080 and height=566'''

    def __init__(self, *args, **kwargs):
        kwargs['width'] = kwargs.get('width', 1080)
        kwargs['height'] = kwargs.get('height', 566)
        super().__init__(*args, **kwargs)



import textwrap

from nider.utils import get_font

from nider.mixins import MultilineTextMixin
from nider.mixins import AlignMixin
from nider.colors.utils import color_to_rgb


class Text:
    '''Base class for the text

    Attributes:
        text (str): text used in the object.
        fontfullpath (str): path to the font used in the object.
        fontsize (int): size of the font.
        color (str, tuple): either hex or rgb representation of text color.
        drop_shadow (bool): boolean flag that indicates if text has to drop shadow.
        shadowcolor (str, tuple): either hex or rgb representation of shadowcolor color.
    '''

    def __init__(self, text, fontfullpath,
                 fontsize, color, drop_shadow, shadowcolor):
        self._set_text(text)
        self.fontfullpath = fontfullpath
        self.fontsize = fontsize
        self._set_font()
        self._set_decoration(color, drop_shadow, shadowcolor)

    def _set_text(self, text):
        '''Sets object's text data'''
        self.text = text

    def _set_font(self):
        '''Sets object's font'''
        self.font = get_font(self.fontfullpath, self.fontsize)

    def _set_decoration(self, color, drop_shadow, shadowcolor):
        '''Sets object's color and shadow'''
        self.color = color_to_rgb(color)
        if drop_shadow is True:
            self.drop_shadow = drop_shadow
            self.shadowcolor = color_to_rgb(shadowcolor)
        else:
            self.drop_shadow = False
            self.shadowcolor = None


class MultilineText(MultilineTextMixin, Text):
    '''Base class for the multiline text'''

    def __init__(self, text_width, line_padding, *args, **kwargs):
        MultilineTextMixin.__init__(
            self, text_width=text_width, line_padding=line_padding)
        Text.__init__(self, *args, **kwargs)


class SingleLineTextUnit(AlignMixin, Text):
    '''Base class for the single line text unit'''

    def __init__(self, text, fontfullpath,
                 fontsize=18, align='right',
                 color='#000', drop_shadow=False, shadowcolor='#646464'
                 ):
        AlignMixin.__init__(self, align=align)
        Text.__init__(
            self, text=text, fontfullpath=fontfullpath, fontsize=fontsize,
            color=color, drop_shadow=drop_shadow,
            shadowcolor=shadowcolor
        )
        self._set_height()

    def _set_height(self):
        '''Sets unit\'s height'''
        _, self.height = self.font.getsize(self.text)


class MultilineTextUnit(AlignMixin, MultilineText):
    '''Base class for the multiline text unit'''

    def __init__(self, text,
                 fontfullpath, fontsize=18,
                 text_width=21, line_padding=6,
                 color='#000', drop_shadow=False, shadowcolor='#646464',
                 align='center'):
        AlignMixin.__init__(self, align=align)
        MultilineText.__init__(self, text=text,
                               fontfullpath=fontfullpath, fontsize=fontsize,
                               text_width=text_width, line_padding=line_padding,
                               color=color, drop_shadow=drop_shadow,
                               shadowcolor=shadowcolor
                               )
        self._set_unit()

    def _set_unit(self):
        '''Sets a unit used in the image

        Sets textwraped unit's text that will be used in the image and also
        attachs header height to the obj instance
        '''
        self.wrapped_lines = textwrap.wrap(self.text, width=self.text_width)
        self._set_height()

    def _set_height(self):
        '''Sets unit's height used in the image

        Calculates unit's height by adding height of each line and its padding
        '''
        self.height = 0
        for line in self.wrapped_lines:
            _, h = self.font.getsize(line)
            self.height += h

        self.height += (len(self.wrapped_lines) - 1) * self.line_padding



import unittest

from unittest import mock

from nider.core import Text
from nider.core import MultilineText
from nider.core import SingleLineTextUnit
from nider.core import MultilineTextUnit


class TestText(unittest.TestCase):

    def setUp(self):
        self.text_mock = mock.Mock()

    # Tests for text handling

    def test_setting_text(self):
        Text._set_text(self.text_mock, 'foo')
        self.assertEqual(self.text_mock.text, 'foo')

    def test_setting_font(self):
        self.text_mock.fontfullpath = 'foo/bar'
        self.text_mock.fontsize = 23
        Text._set_font(self.text_mock)
        self.assertIsNotNone(self.text_mock.font)

    def test_setting_decoration_with_drop_shadow(self):
        Text._set_decoration(self.text_mock, (0, 0, 0), True, (0, 0, 0))
        self.assertEqual(self.text_mock.color, (0, 0, 0))
        self.assertEqual(self.text_mock.drop_shadow, True)
        self.assertEqual(self.text_mock.shadowcolor, (0, 0, 0))

    def test_setting_decoration_without_drop_shadow(self):
        Text._set_decoration(self.text_mock, (0, 0, 0), False, (0, 0, 0))
        self.assertEqual(self.text_mock.color, (0, 0, 0))
        self.assertEqual(self.text_mock.drop_shadow, False)
        self.assertIsNone(self.text_mock.shadowcolor)


class TestMultilineText(unittest.TestCase):

    @mock.patch('nider.mixins.MultilineTextMixin.__init__')
    @mock.patch('nider.core.Text.__init__')
    def test_proper_inheritance(self,
                                T_mock,
                                MTM_mock):
        multiline_text = MultilineText(text='foo', fontfullpath='foo.ttf',
                                       fontsize=15, color=(0, 0, 0),
                                       drop_shadow=False, shadowcolor=(0, 0, 0),
                                       text_width=20, line_padding=5)
        T_mock.assert_called_once_with(multiline_text, text='foo',
                                       fontfullpath='foo.ttf', fontsize=15,
                                       color=(0, 0, 0),
                                       drop_shadow=False, shadowcolor=(0, 0, 0))
        MTM_mock.assert_called_once_with(multiline_text,
                                         text_width=20,
                                         line_padding=5)


class TestSingleLineTextUnit(unittest.TestCase):

    @mock.patch('nider.core.SingleLineTextUnit._set_height')
    @mock.patch('nider.mixins.AlignMixin.__init__')
    @mock.patch('nider.core.Text.__init__')
    def test_proper_inheritance(self, T_mock, AM_mock, _set_height_mock):

        unit = SingleLineTextUnit(text='foo',
                                  fontfullpath=None, fontsize=18,
                                  color='#000', drop_shadow=False,
                                  shadowcolor='#646464', align='center')
        T_mock.assert_called_once_with(unit, text='foo',
                                       fontfullpath=None, fontsize=18,
                                       color='#000', drop_shadow=False,
                                       shadowcolor='#646464'
                                       )
        AM_mock.assert_called_once_with(unit, align='center')

    def test_set_height(self):
        unit = SingleLineTextUnit('foo', None)
        # height of the default font
        self.assertEqual(unit.height, 11)


class TestMultilineTextUnit(unittest.TestCase):

    @mock.patch('nider.core.MultilineTextUnit._set_unit')
    @mock.patch('nider.mixins.AlignMixin.__init__')
    @mock.patch('nider.core.MultilineText.__init__')
    def test_proper_inheritance(self,
                                MT_mock,
                                AM_mock,
                                _set_unit_mock):
        unit = MultilineTextUnit(text='foo', fontfullpath='foo.ttf',
                                 fontsize=15, color=(0, 0, 0),
                                 drop_shadow=False, shadowcolor=(0, 0, 0),
                                 text_width=20, line_padding=5,
                                 align='center')
        MT_mock.assert_called_once_with(unit, text='foo',
                                        fontfullpath='foo.ttf', fontsize=15,
                                        color=(0, 0, 0),
                                        drop_shadow=False, shadowcolor=(0, 0, 0),
                                        text_width=20,
                                        line_padding=5)
        AM_mock.assert_called_once_with(unit, align='center')

    @mock.patch('nider.core.MultilineTextUnit._set_height')
    def test_set_unit(self, _set_height_mock):
        unit = MultilineTextUnit(text='Lorem imsup dolor si amet',
                                 fontfullpath=None,
                                 text_width=5)
        self.assertEqual(unit.wrapped_lines,
                         ['Lorem', 'imsup', 'dolor', 'si', 'amet'])

    def test_set_height(self):
        unit = MultilineTextUnit(text='Lorem imsup dolor si amet',
                                 fontfullpath=None,
                                 text_width=5,
                                 line_padding=6)
        self.assertEqual(unit.height, 79)


if __name__ == '__main__':
    unittest.main()



import os
import warnings
import random

from contextlib import contextmanager

from PIL import Image
from PIL import ImageFont

from nider.colors import FLAT_UI_COLORS

from nider.exceptions import DefaultFontWarning
from nider.exceptions import FontNotFoundWarning


def get_font(fontfullpath, fontsize):
    '''Returns a PIL ImageFont object

    Attributes:
        fontfullpath (str): path to the desired font
        fontsize (int): size of the font
    '''
    if fontfullpath is None:
        warnings.warn(DefaultFontWarning())
        return ImageFont.load_default()
    elif not os.path.exists(fontfullpath):
        warnings.warn(FontNotFoundWarning(fontfullpath))
        return ImageFont.load_default()
    return ImageFont.truetype(fontfullpath, fontsize)


def is_path_creatable(pathname):
    '''
    `True` if the current user has sufficient permissions to create the passed
    pathname; `False` otherwise.
    '''
    # Parent directory of the passed path. If empty, we substitute the current
    # working directory (CWD) instead.
    dirname = os.path.dirname(pathname) or os.getcwd()
    return os.access(dirname, os.W_OK)


@contextmanager
def create_test_image():
    try:
        image = Image.new('RGBA', size=(50, 50), color=(155, 0, 0))
        image.save('test.png')
        yield
    finally:
        os.remove('test.png')


def get_random_texture():
    '''Returns the path to a random texture from the local /textures folder'''
    textures_folder = os.path.dirname(
        os.path.realpath(__file__)) + '/textures'
    texture = random.choice(os.listdir(textures_folder))
    texture_path = textures_folder + '/' + texture
    return texture_path


def get_random_bgcolor():
    '''Returns random FLAT_UI_COLOR'''
    return random.choice(list(FLAT_UI_COLORS.values()))



import math
import os
import unittest

from unittest import mock

from PIL import Image as PIL_Image
from PIL import ImageDraw

from nider.core import MultilineTextUnit

from nider.models import Content
from nider.models import FacebookLandscapePost
from nider.models import FacebookSquarePost
from nider.models import Header
from nider.models import Image
from nider.models import InstagramLandscapePost
from nider.models import InstagramPortraitPost
from nider.models import InstagramSquarePost
from nider.models import Linkback
from nider.models import Paragraph
from nider.models import TwitterLargeCard
from nider.models import TwitterPost


from nider.utils import create_test_image
from nider.utils import get_random_texture

from nider.exceptions import ImageSizeFixedWarning


class TestLinkback(unittest.TestCase):

    def setUp(self):
        self.linkback = Linkback(text='foo', fontfullpath=None)

    def test_set_height(self):
        self.assertEqual(self.linkback.height, 31)


class TestContent(unittest.TestCase):

    def setUp(self):
        header = Header(text='foo', fontfullpath=None)
        linkback = Linkback(text='bar', fontfullpath=None)
        para = Paragraph(text='foo bar', fontfullpath=None)
        self.content = Content(para, header=header, linkback=linkback)

    def test_content_height(self):
        self.assertIsNotNone(self.content.height)


class TestImageInitializationMethods(unittest.TestCase):

    def setUp(self):
        self.image_mock = mock.Mock()

    def test_set_content(self):
        para = Paragraph(text='foo bar', fontfullpath=None)
        content = Content(para)
        Image._set_content(self.image_mock, content)
        self.assertEqual(self.image_mock.content, content)

    def test_set_fullpath_with_valid_path(self):
        fullpath = 'test.png'
        Image._set_fullpath(self.image_mock, fullpath)
        self.assertEqual(self.image_mock.fullpath, fullpath)

    def test_set_fullpath_with_invalid_path(self):
        fullpath = 'non/existent/directory/test.png'
        with self.assertRaises(AttributeError):
            Image._set_fullpath(self.image_mock, fullpath)
        self.assertNotEqual(self.image_mock.fullpath, fullpath)

    def test_set_image_size(self):
        w, h = 500, 500
        Image._set_image_size(self.image_mock, w, h)
        self.assertEqual(self.image_mock.width, w)
        self.assertEqual(self.image_mock.height, h)

    def test_invalid_params_to_set_image_size(self):
        invalid_sizes = [(-1, -2), (-1.5, -5.5)]
        for invalid_size in invalid_sizes:
            with self.subTest():
                with self.assertRaises(AttributeError):
                    Image._set_image_size(self.image_mock, invalid_size[
                        0], invalid_size[1])


class TestImageBaseMethods(unittest.TestCase):

    def setUp(self):
        header = Header(text='foo', fontfullpath=None)
        linkback = Linkback(text='bar', fontfullpath=None)
        para = Paragraph(text='foo bar', fontfullpath=None)
        content = Content(para, header=header, linkback=linkback)
        fullpath = 'test.png'
        self.img = Image(content, fullpath)

    def test_fix_img_size(self):
        self.img.content.height = 101
        self.img.height = 100
        with self.assertWarns(ImageSizeFixedWarning):
            self.img._fix_image_size()
        self.assertFalse(self.img.content.fits)
        self.assertEqual(self.img.height, self.img.content.height)

    def test_no_need_for_fix_img_size(self):
        self.img.content.height = 100
        self.img.height = 101
        self.img._fix_image_size()
        self.assertTrue(self.img.content.fits)
        self.assertNotEqual(self.img.height, self.img.content.height)

    def test_create_img(self):
        self.img._create_image()
        self.assertIsInstance(self.img.image, PIL_Image.Image)

    def test_create_draw_object(self):
        self.img._create_image()
        self.img._create_draw_object()
        self.assertIsInstance(self.img.draw, ImageDraw.ImageDraw)

    @mock.patch('nider.models.Image._save')
    @mock.patch('nider.models.Image._draw_content')
    def test_draw_on_texture(self,
                             _draw_content_mock,
                             _save):
        self.img.draw_on_texture()
        self.assertTrue(_draw_content_mock.called)

    def test_draw_on_texture_with_invalid_texturepath(self):
        with self.assertRaises(FileNotFoundError):
            self.img.draw_on_texture(texture_path='foo/bar.png')

    @mock.patch('nider.models.Image._save')
    @mock.patch('nider.models.Image._draw_content')
    def test_draw_on_bg(self,
                        _draw_content_mock,
                        _save):
        self.img.draw_on_bg()
        self.assertTrue(_draw_content_mock.called)

    @mock.patch('nider.models.Image._save')
    @mock.patch('nider.models.Image._draw_content')
    def test_draw_on_image(self,
                           _draw_content_mock,
                           _save):
        with create_test_image():
            self.img.draw_on_image(
                image_path=os.path.abspath('test.png'))
        self.assertTrue(_draw_content_mock.called)

    def test_draw_on_image_with_invalid_imagepath(self):
        with self.assertRaises(FileNotFoundError):
            self.img.draw_on_image('foo/bar.png')


class TestImageMethodsThatRequireImageAndDraw(unittest.TestCase):

    def setUp(self):
        header = Header(text='foo', fontfullpath=None)
        linkback = Linkback(text='bar', fontfullpath=None)
        para = Paragraph(text='foo bar', fontfullpath=None)
        content = Content(para, header=header, linkback=linkback, padding=45)
        self.fullpath = 'test.png'
        self.img = Image(content, self.fullpath)
        self.img._create_image()
        self.img._create_draw_object()

    @classmethod
    def tearDownClass(self):
        fullpath = 'test.png'
        os.remove(fullpath)

    @mock.patch('PIL.Image.Image.paste')
    def test_fill_img_with_texture(self, mock):
        texture = get_random_texture()
        self.img._fill_image_with_texture(texture)
        self.assertTrue(mock.called)

    @mock.patch('PIL.ImageDraw.ImageDraw.rectangle')
    def test_fill_img_with_color(self, mock):
        bgcolor = (0, 0, 0)
        self.img.bgcolor = (0, 0, 0)
        self.img._fill_image_with_color()
        mock.assert_called_once_with(
            [(0, 0), self.img.image.size], fill=bgcolor)

    @mock.patch('nider.models.Image._draw_unit')
    def test_draw_header(self, mock):
        self.img._draw_header()
        mock.assert_called_once_with(45,
                                     self.img.header)

    @mock.patch('nider.models.Image._draw_unit')
    def test_draw_para_with_content_that_fits(self, mock):
        self.img._draw_para()
        current_h = math.floor(
            (self.img.height - self.img.para.height) / 2)
        mock.assert_called_once_with(current_h, self.img.para)

    @mock.patch('nider.models.Image._draw_unit')
    def test_draw_para_content_with_header_that_does_not_fit(self, mock):
        self.img.content.fits = False
        self.img._draw_para()
        header_with_padding_height = 2 * \
            self.img.content.padding + self.img.header.height
        current_h = header_with_padding_height
        mock.assert_called_once_with(current_h, self.img.para)

    @mock.patch('nider.models.Image._draw_unit')
    def test_draw_para_content_without_header_that_does_not_fit(self, mock):
        self.img.content.fits = False
        self.img.header = None
        self.img._draw_para()
        current_h = self.img.content.padding
        mock.assert_called_once_with(current_h, self.img.para)

    @mock.patch('PIL.ImageDraw.ImageDraw.text')
    def test_draw_linkback(self, mock):
        self.img.color = (0, 0, 0)
        aligns = ['center', 'right', 'left']
        for align in aligns:
            with self.subTest():
                self.img.linkback.align = align
                self.img._draw_linkback()
                self.assertTrue(mock.called)

    @mock.patch('nider.models.Image._draw_linkback')
    @mock.patch('nider.models.Image._draw_para')
    @mock.patch('nider.models.Image._draw_header')
    def test_draw_content(self, _draw_header_mock,
                          _draw_para_mock, _draw_linkback_mock):
        self.img._draw_content()
        self.assertTrue(_draw_header_mock.called)
        self.assertTrue(_draw_para_mock.called)
        self.assertTrue(_draw_linkback_mock.called)

    @mock.patch('PIL.ImageDraw.ImageDraw.text')
    def test_draw_unit(self, text_mock):
        drop_shadow_options = [True, False]
        self.img.color = (0, 0, 0)
        start_height = 0
        aligns = ['center', 'right', 'left']
        for align in aligns:
            for drop_shadow_option in drop_shadow_options:
                self.img.drop_shadow = drop_shadow_option
                if drop_shadow_option:
                    self.img.shadowcolor = (0, 0, 0)
                with self.subTest():
                    unit = MultilineTextUnit(
                        text='foo', fontfullpath=None, drop_shadow=drop_shadow_option,
                        align=align)
                    self.img._draw_unit(start_height, unit)
                    self.assertTrue(text_mock.called)

    def test_save(self):
        self.img._save()
        self.assertTrue(os.path.isfile(self.fullpath))


class TestFacebookSquarePost(unittest.TestCase):

    @mock.patch('nider.models.Image._set_fullpath')
    @mock.patch('nider.models.Image._set_content')
    def setUp(self, *mocks):
        self.post = FacebookSquarePost(content=mock.Mock(),
                                       fullpath=mock.Mock())

    def test_size(self):
        self.assertEqual(self.post.width, 470)
        self.assertEqual(self.post.height, 470)


class TestFacebookLandscapePost(unittest.TestCase):

    @mock.patch('nider.models.Image._set_fullpath')
    @mock.patch('nider.models.Image._set_content')
    def setUp(self, *mocks):
        self.post = FacebookLandscapePost(content=mock.Mock(),
                                          fullpath=mock.Mock())

    def test_size(self):
        self.assertEqual(self.post.width, 1024)
        self.assertEqual(self.post.height, 512)


class TestTwitterPost(unittest.TestCase):

    @mock.patch('nider.models.Image._set_fullpath')
    @mock.patch('nider.models.Image._set_content')
    def setUp(self, *mocks):
        self.post = TwitterPost(content=mock.Mock(),
                                fullpath=mock.Mock())

    def test_size(self):
        self.assertEqual(self.post.width, 1024)
        self.assertEqual(self.post.height, 512)


class TestTwitterLargeCard(unittest.TestCase):

    @mock.patch('nider.models.Image._set_fullpath')
    @mock.patch('nider.models.Image._set_content')
    def setUp(self, *mocks):
        self.post = TwitterLargeCard(content=mock.Mock(),
                                     fullpath=mock.Mock())

    def test_size(self):
        self.assertEqual(self.post.width, 506)
        self.assertEqual(self.post.height, 506)


class TestInstagramSquarePost(unittest.TestCase):

    @mock.patch('nider.models.Image._set_fullpath')
    @mock.patch('nider.models.Image._set_content')
    def setUp(self, *mocks):
        self.post = InstagramSquarePost(content=mock.Mock(),
                                        fullpath=mock.Mock())

    def test_size(self):
        self.assertEqual(self.post.width, 1080)
        self.assertEqual(self.post.height, 1080)


class TestInstagramPortraitPost(unittest.TestCase):

    @mock.patch('nider.models.Image._set_fullpath')
    @mock.patch('nider.models.Image._set_content')
    def setUp(self, *mocks):
        self.post = InstagramPortraitPost(content=mock.Mock(),
                                          fullpath=mock.Mock())

    def test_size(self):
        self.assertEqual(self.post.width, 1080)
        self.assertEqual(self.post.height, 1350)


class TestInstagramLandscapePost(unittest.TestCase):

    @mock.patch('nider.models.Image._set_fullpath')
    @mock.patch('nider.models.Image._set_content')
    def setUp(self, *mocks):
        self.post = InstagramLandscapePost(content=mock.Mock(),
                                           fullpath=mock.Mock())

    def test_size(self):
        self.assertEqual(self.post.width, 1080)
        self.assertEqual(self.post.height, 566)


if __name__ == '__main__':
    unittest.main()
